package sun.management.snmp.jvminstr;

import com.sun.jmx.snmp.SnmpStatusException;
import java.lang.management.ManagementFactory;
import java.lang.management.ThreadMXBean;
import javax.management.MBeanServer;
import com.sun.jmx.snmp.agent.SnmpMib;
import sun.management.snmp.util.MibLogger;
import sun.management.snmp.jvmmib.EnumJvmThreadContentionMonitoring;
import sun.management.snmp.jvmmib.EnumJvmThreadCpuTimeMonitoring;
import sun.management.snmp.jvmmib.JvmThreadingMBean;

public class JvmThreadingImpl implements JvmThreadingMBean
{
    static final EnumJvmThreadCpuTimeMonitoring JvmThreadCpuTimeMonitoringUnsupported;
    static final EnumJvmThreadCpuTimeMonitoring JvmThreadCpuTimeMonitoringEnabled;
    static final EnumJvmThreadCpuTimeMonitoring JvmThreadCpuTimeMonitoringDisabled;
    static final EnumJvmThreadContentionMonitoring JvmThreadContentionMonitoringUnsupported;
    static final EnumJvmThreadContentionMonitoring JvmThreadContentionMonitoringEnabled;
    static final EnumJvmThreadContentionMonitoring JvmThreadContentionMonitoringDisabled;
    private long jvmThreadPeakCountReset;
    static final MibLogger log;
    
    public JvmThreadingImpl(SnmpMib snmpMib) {
        this.jvmThreadPeakCountReset = 0L;
        JvmThreadingImpl.log.debug("JvmThreadingImpl", "Constructor");
    }
    
    public JvmThreadingImpl(SnmpMib snmpMib, MBeanServer mBeanServer) {
        this.jvmThreadPeakCountReset = 0L;
        JvmThreadingImpl.log.debug("JvmThreadingImpl", "Constructor with server");
    }
    
    static ThreadMXBean getThreadMXBean() {
        return ManagementFactory.getThreadMXBean();
    }
    
    public EnumJvmThreadCpuTimeMonitoring getJvmThreadCpuTimeMonitoring() throws SnmpStatusException {
        ThreadMXBean threadMXBean = getThreadMXBean();
        if (!threadMXBean.isThreadCpuTimeSupported()) {
            JvmThreadingImpl.log.debug("getJvmThreadCpuTimeMonitoring", "Unsupported ThreadCpuTimeMonitoring");
            return JvmThreadingImpl.JvmThreadCpuTimeMonitoringUnsupported;
        }
        try {
            if (threadMXBean.isThreadCpuTimeEnabled()) {
                JvmThreadingImpl.log.debug("getJvmThreadCpuTimeMonitoring", "Enabled ThreadCpuTimeMonitoring");
                return JvmThreadingImpl.JvmThreadCpuTimeMonitoringEnabled;
            }
            JvmThreadingImpl.log.debug("getJvmThreadCpuTimeMonitoring", "Disabled ThreadCpuTimeMonitoring");
            return JvmThreadingImpl.JvmThreadCpuTimeMonitoringDisabled;
        }
        catch (UnsupportedOperationException ex) {
            JvmThreadingImpl.log.debug("getJvmThreadCpuTimeMonitoring", "Newly unsupported ThreadCpuTimeMonitoring");
            return JvmThreadingImpl.JvmThreadCpuTimeMonitoringUnsupported;
        }
    }
    
    public void setJvmThreadCpuTimeMonitoring(EnumJvmThreadCpuTimeMonitoring enumJvmThreadCpuTimeMonitoring) throws SnmpStatusException {
        ThreadMXBean threadMXBean = getThreadMXBean();
        if (JvmThreadingImpl.JvmThreadCpuTimeMonitoringEnabled.intValue() == enumJvmThreadCpuTimeMonitoring.intValue()) {
            threadMXBean.setThreadCpuTimeEnabled(true);
        }
        else {
            threadMXBean.setThreadCpuTimeEnabled(false);
        }
    }
    
    public void checkJvmThreadCpuTimeMonitoring(EnumJvmThreadCpuTimeMonitoring enumJvmThreadCpuTimeMonitoring) throws SnmpStatusException {
        if (JvmThreadingImpl.JvmThreadCpuTimeMonitoringUnsupported.intValue() == enumJvmThreadCpuTimeMonitoring.intValue()) {
            JvmThreadingImpl.log.debug("checkJvmThreadCpuTimeMonitoring", "Try to set to illegal unsupported value");
            throw new SnmpStatusException(10);
        }
        if (JvmThreadingImpl.JvmThreadCpuTimeMonitoringEnabled.intValue() == enumJvmThreadCpuTimeMonitoring.intValue() || JvmThreadingImpl.JvmThreadCpuTimeMonitoringDisabled.intValue() == enumJvmThreadCpuTimeMonitoring.intValue()) {
            if (getThreadMXBean().isThreadCpuTimeSupported()) {
                return;
            }
            JvmThreadingImpl.log.debug("checkJvmThreadCpuTimeMonitoring", "Unsupported operation, can't set state");
            throw new SnmpStatusException(12);
        }
        else {
            JvmThreadingImpl.log.debug("checkJvmThreadCpuTimeMonitoring", "unknown enum value ");
            throw new SnmpStatusException(10);
        }
    }
    
    public EnumJvmThreadContentionMonitoring getJvmThreadContentionMonitoring() throws SnmpStatusException {
        ThreadMXBean threadMXBean = getThreadMXBean();
        if (!threadMXBean.isThreadContentionMonitoringSupported()) {
            JvmThreadingImpl.log.debug("getJvmThreadContentionMonitoring", "Unsupported ThreadContentionMonitoring");
            return JvmThreadingImpl.JvmThreadContentionMonitoringUnsupported;
        }
        if (threadMXBean.isThreadContentionMonitoringEnabled()) {
            JvmThreadingImpl.log.debug("getJvmThreadContentionMonitoring", "Enabled ThreadContentionMonitoring");
            return JvmThreadingImpl.JvmThreadContentionMonitoringEnabled;
        }
        JvmThreadingImpl.log.debug("getJvmThreadContentionMonitoring", "Disabled ThreadContentionMonitoring");
        return JvmThreadingImpl.JvmThreadContentionMonitoringDisabled;
    }
    
    public void setJvmThreadContentionMonitoring(EnumJvmThreadContentionMonitoring enumJvmThreadContentionMonitoring) throws SnmpStatusException {
        ThreadMXBean threadMXBean = getThreadMXBean();
        if (JvmThreadingImpl.JvmThreadContentionMonitoringEnabled.intValue() == enumJvmThreadContentionMonitoring.intValue()) {
            threadMXBean.setThreadContentionMonitoringEnabled(true);
        }
        else {
            threadMXBean.setThreadContentionMonitoringEnabled(false);
        }
    }
    
    public void checkJvmThreadContentionMonitoring(EnumJvmThreadContentionMonitoring enumJvmThreadContentionMonitoring) throws SnmpStatusException {
        if (JvmThreadingImpl.JvmThreadContentionMonitoringUnsupported.intValue() == enumJvmThreadContentionMonitoring.intValue()) {
            JvmThreadingImpl.log.debug("checkJvmThreadContentionMonitoring", "Try to set to illegal unsupported value");
            throw new SnmpStatusException(10);
        }
        if (JvmThreadingImpl.JvmThreadContentionMonitoringEnabled.intValue() == enumJvmThreadContentionMonitoring.intValue() || JvmThreadingImpl.JvmThreadContentionMonitoringDisabled.intValue() == enumJvmThreadContentionMonitoring.intValue()) {
            if (getThreadMXBean().isThreadContentionMonitoringSupported()) {
                return;
            }
            JvmThreadingImpl.log.debug("checkJvmThreadContentionMonitoring", "Unsupported operation, can't set state");
            throw new SnmpStatusException(12);
        }
        else {
            JvmThreadingImpl.log.debug("checkJvmThreadContentionMonitoring", "Try to set to unknown value");
            throw new SnmpStatusException(10);
        }
    }
    
    public Long getJvmThreadTotalStartedCount() throws SnmpStatusException {
        return new Long(getThreadMXBean().getTotalStartedThreadCount());
    }
    
    public Long getJvmThreadPeakCount() throws SnmpStatusException {
        return new Long((long)getThreadMXBean().getPeakThreadCount());
    }
    
    public Long getJvmThreadDaemonCount() throws SnmpStatusException {
        return new Long((long)getThreadMXBean().getDaemonThreadCount());
    }
    
    public Long getJvmThreadCount() throws SnmpStatusException {
        return new Long((long)getThreadMXBean().getThreadCount());
    }
    
    public synchronized Long getJvmThreadPeakCountReset() throws SnmpStatusException {
        return new Long(this.jvmThreadPeakCountReset);
    }
    
    public synchronized void setJvmThreadPeakCountReset(Long n) throws SnmpStatusException {
        if ((long)n > this.jvmThreadPeakCountReset) {
            long currentTimeMillis = System.currentTimeMillis();
            getThreadMXBean().resetPeakThreadCount();
            this.jvmThreadPeakCountReset = currentTimeMillis;
            JvmThreadingImpl.log.debug("setJvmThreadPeakCountReset", "jvmThreadPeakCountReset=" + currentTimeMillis);
        }
    }
    
    public void checkJvmThreadPeakCountReset(Long n) throws SnmpStatusException {
    }
    
    static {
        JvmThreadCpuTimeMonitoringUnsupported = new EnumJvmThreadCpuTimeMonitoring("unsupported");
        JvmThreadCpuTimeMonitoringEnabled = new EnumJvmThreadCpuTimeMonitoring("enabled");
        JvmThreadCpuTimeMonitoringDisabled = new EnumJvmThreadCpuTimeMonitoring("disabled");
        JvmThreadContentionMonitoringUnsupported = new EnumJvmThreadContentionMonitoring("unsupported");
        JvmThreadContentionMonitoringEnabled = new EnumJvmThreadContentionMonitoring("enabled");
        JvmThreadContentionMonitoringDisabled = new EnumJvmThreadContentionMonitoring("disabled");
        log = new MibLogger(JvmThreadingImpl.class);
    }
}
